/*
  printCallbacks.c
  By Tuxgasy <tuxgasy@tuxfamily.org>
  
  This file is part of Glda.
  
  You may redistribute it and/or modify it under the terms of the
  GNU General Public License, as published by the Free Software
  Foundation; either version 2 of the License, or (at your option) any later version.
  
  Glda is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  See the GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with Glda; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <libxml/tree.h>
#include <libxml/parser.h>
#include <libxml/xpath.h>
#include <glib.h>
#include <gtk/gtk.h>
#include <math.h>

#include "main.h"
#include "interface.h"

void begin_print(GtkPrintOperation *operation, GtkPrintContext *context, Print *pPrintData){
  gdouble height;
  gint NbArt;
  gdouble size_police;
  gdouble PrixTot;
  
  /* Nombre d'articles et prix total */
  getNbArticlesAndPrixTotal(pPrintData->pGlda->pLdaChainee, pPrintData->pGlda->pMagasin, &NbArt, &PrixTot);
  /* Taille police */
  size_police = g_key_file_get_double(pPrintData->pGlda->pConfig, "print", "size_police", NULL);
  /* Hauteur page */
  height = gtk_print_context_get_height(context) - HEADER_HEIGHT - HEADER_GAP - 2 * size_police;
  /* nombre d'articles par page */
	pPrintData->number_articles_per_page = floor(height / (2 * size_police));
	/* nombre d'articles */
  pPrintData->number_articles = NbArt;
  pPrintData->number_articles_differents = getNbArticlesDifferents(pPrintData->pGlda->pLdaChainee);
  /* nombre de pages */
  pPrintData->total_pages = pPrintData->number_articles_differents / pPrintData->number_articles_per_page + 1;
  
  gtk_print_operation_set_n_pages(operation, pPrintData->total_pages);
}

void draw_page(GtkPrintOperation *operation, GtkPrintContext *context, gint page_nr, Print *pPrintData){
  cairo_t *cr;
  PangoLayout *layout;
  gdouble width, text_height;
  gint number_article, i, text_width, layout_height;
  PangoFontDescription *desc;
  gchar *page_str;
  gchar *pText;
  gdouble Prix;
  gint Quantite, length_line, length_maxi_number, length_maxi_name, length_maxi_price, length_maxi_quantity, cellpadding;
  gdouble size_police, border;
  gboolean show_title, show_page, show_column_number, show_column_name, show_column_price, show_column_quantity, show_line_total;
  gdouble *color;
  gsize length;
  
  cr = gtk_print_context_get_cairo_context(context);
  width = gtk_print_context_get_width(context);
  layout = gtk_print_context_create_pango_layout(context);
  size_police = g_key_file_get_double(pPrintData->pGlda->pConfig, "print", "size_police", NULL);
  desc = pango_font_description_from_string(g_key_file_get_string(pPrintData->pGlda->pConfig, "print", "police", NULL));
  pango_font_description_set_size(desc, size_police * PANGO_SCALE);

  pango_layout_get_size(layout, NULL, &layout_height);
  text_height = (gdouble)layout_height / PANGO_SCALE;
    
  pango_layout_set_font_description(layout, desc);
  
  show_title = g_key_file_get_boolean(pPrintData->pGlda->pConfig, "print", "show_title", NULL);
  show_page = g_key_file_get_boolean(pPrintData->pGlda->pConfig, "print", "show_page", NULL);
  color = g_key_file_get_double_list(pPrintData->pGlda->pConfig, "print", "color", &length, NULL);
  border = g_key_file_get_double(pPrintData->pGlda->pConfig, "print", "border", NULL);
  show_column_number = g_key_file_get_boolean(pPrintData->pGlda->pConfig, "print", "show_column_number", NULL);
  show_column_name = g_key_file_get_boolean(pPrintData->pGlda->pConfig, "print", "show_column_name", NULL);
  show_column_price = g_key_file_get_boolean(pPrintData->pGlda->pConfig, "print", "show_column_price", NULL);
  show_column_quantity = g_key_file_get_boolean(pPrintData->pGlda->pConfig, "print", "show_column_quantity", NULL);
  length_maxi_number = g_key_file_get_integer(pPrintData->pGlda->pConfig, "print", "length_maxi_number", NULL);
  length_maxi_name = g_key_file_get_integer(pPrintData->pGlda->pConfig, "print", "length_maxi_name", NULL);
  length_maxi_price = g_key_file_get_integer(pPrintData->pGlda->pConfig, "print", "length_maxi_price", NULL);
  length_maxi_quantity = g_key_file_get_integer(pPrintData->pGlda->pConfig, "print", "length_maxi_quantity", NULL);
  show_line_total = g_key_file_get_boolean(pPrintData->pGlda->pConfig, "print", "show_line_total", NULL);
  cellpadding = g_key_file_get_integer(pPrintData->pGlda->pConfig, "print", "cellpadding", NULL);
  
   /* Couleur des lignes (noir) */
  cairo_set_source_rgb(cr, color[0], color[1], color[2]);
  /* Taille des lignes */
	cairo_set_line_width(cr, border);
	
  /* Titre */
  if(show_title){
	  pText = g_strdup_printf("Glda v%s", pPrintData->pGlda->version);
  	pango_layout_set_text(layout, pText, -1);
  	g_free(pText);
  	pango_layout_set_width(layout, -1);
  	pango_layout_set_alignment(layout, PANGO_ALIGN_LEFT);
  
  	cairo_move_to(cr, 0, (HEADER_HEIGHT - text_height) / 2);
  	pango_cairo_show_layout(cr, layout);
  }
  
  /* Page */
  if(show_page){
	  page_str = g_strdup_printf("%d / %d", page_nr+1, pPrintData->total_pages);
  	pango_layout_set_text(layout, page_str, -1);
  	pango_layout_get_size(layout, &text_width, NULL);
  	pango_layout_set_alignment(layout, PANGO_ALIGN_RIGHT);
  	g_free(page_str);
  	
  	cairo_move_to(cr, width - (text_width / PANGO_SCALE), (HEADER_HEIGHT - text_height) / 2);
  	pango_cairo_show_layout(cr, layout);
  }
	
	/* Calcul length line */
	length_line = 0;
	if(show_column_number)
		length_line = length_line + length_maxi_number + 1;
	if(show_column_name)
		length_line = length_line + length_maxi_name + 1;
	if(show_column_price)
		length_line = length_line + length_maxi_price + 1;
	if(show_column_quantity)
		length_line = length_line + length_maxi_quantity + 1;

	/* 1ère ligne */
	cairo_move_to(cr, 0, HEADER_HEIGHT + HEADER_GAP);
	cairo_line_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP);

  cairo_move_to(cr, 0, HEADER_HEIGHT + HEADER_GAP);
	cairo_line_to(cr, 0, HEADER_HEIGHT + HEADER_GAP + 2 * size_police);
	
	length_line = 0;
	
	/* Colonne Numéro */
	if(show_column_number){
		cairo_move_to(cr, size_police, HEADER_HEIGHT + HEADER_GAP + cellpadding);
		pango_layout_set_text(layout, "N°", length_maxi_number);
		pango_cairo_show_layout(cr, layout);
		
		length_line = length_line + length_maxi_number + 1;
		
		cairo_move_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP);
		cairo_line_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + 2 * size_police);
	}
	
	/* Colonne Article */
	if(show_column_name){
		cairo_move_to(cr, (length_line + 1) * size_police, HEADER_HEIGHT + HEADER_GAP + cellpadding);
		pango_layout_set_text(layout, "Article(s)", length_maxi_name);
		pango_cairo_show_layout(cr, layout);
		
		length_line = length_line + length_maxi_name + 1;
		
		cairo_move_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP);
		cairo_line_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + 2 * size_police);
	}
	
	/* Colonne Prix */
	if(show_column_price){
		cairo_move_to(cr, (length_line + 1) * size_police, HEADER_HEIGHT + HEADER_GAP + cellpadding);
		pango_layout_set_text(layout, "Prix", length_maxi_price);
		pango_cairo_show_layout(cr, layout);
		
		length_line = length_line + length_maxi_price + 1;
		
		cairo_move_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP);
		cairo_line_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + 2 * size_police);
	}
	
	/* Colonne Quantité */
	if(show_column_quantity){
		cairo_move_to(cr, (length_line + 1) * size_police, HEADER_HEIGHT + HEADER_GAP + cellpadding);
		pango_layout_set_text(layout, "Quantité", length_maxi_quantity);
		pango_cairo_show_layout(cr, layout);
		
		length_line = length_line + length_maxi_quantity + 1;
		
		cairo_move_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP);
		cairo_line_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + 2 * size_police);
	}
	
	/* Fine ligne */
	cairo_move_to(cr, 0, HEADER_HEIGHT + HEADER_GAP + 2 * size_police);
	cairo_line_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + 2 * size_police);
	
  number_article = page_nr * pPrintData->number_articles_per_page;
  for(i=0;i<pPrintData->number_articles_per_page && number_article<pPrintData->number_articles_differents;i++){
  	length_line = 0;
  	
    /* Debut colonne */
    cairo_move_to(cr, 0, HEADER_HEIGHT + HEADER_GAP + (2 + 2 * i) * size_police);
		cairo_line_to(cr, 0, HEADER_HEIGHT + HEADER_GAP + (4 + 2 * i) * size_police);
		
		/* Colonne Numéro */
		if(show_column_number){
    	cairo_move_to(cr, size_police, HEADER_HEIGHT + HEADER_GAP + (2 + 2 * i) * size_police + cellpadding);
	    pText = g_strdup_printf("%d", number_article+1);
			pango_layout_set_text(layout, pText, length_maxi_number);
			g_free(pText);
			pango_cairo_show_layout(cr, layout);
			
			length_line = length_line + length_maxi_number + 1;
			
			cairo_move_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (2 + 2 * i) * size_police);
			cairo_line_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (4 + 2 * i) * size_police);
		}
		
		/* Colonne Article */
		if(show_column_name){
			cairo_move_to(cr, (length_line + 1) * size_police, HEADER_HEIGHT + HEADER_GAP + (2 + 2 * i) * size_police + cellpadding);
			pText = getArticleToLda(pPrintData->pGlda->pLdaChainee, pPrintData->pGlda->pMagasin, number_article, &Prix, &Quantite);
			pango_layout_set_text(layout, pText, length_maxi_name);
			g_free(pText);
			pango_cairo_show_layout(cr, layout);
			
			length_line = length_line + length_maxi_name + 1;
			
			cairo_move_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (2 + 2 * i) * size_police);
			cairo_line_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (4 + 2 * i) * size_police);
		}
		
		/* Colonne Prix */
		if(show_column_price){
			cairo_move_to(cr, (length_line + 1) * size_police, HEADER_HEIGHT + HEADER_GAP + (2 + 2 * i) * size_police + cellpadding);
			pText = g_strdup_printf("%4.2f", Prix);
			pango_layout_set_text(layout, pText, length_maxi_price);
			g_free(pText);
			pango_cairo_show_layout(cr, layout);
			
			length_line = length_line + length_maxi_price + 1;
			
			cairo_move_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (2 + 2 * i) * size_police);
			cairo_line_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (4 + 2 * i) * size_police);
		}
		
		/* Colonne Quantité */
		if(show_column_quantity){
			cairo_move_to(cr, (length_line + 1) * size_police, HEADER_HEIGHT + HEADER_GAP + (2 + 2 * i) * size_police + cellpadding);
			pText = g_strdup_printf("%d", Quantite);
			pango_layout_set_text(layout, pText, length_maxi_quantity);
			g_free(pText);
			pango_cairo_show_layout(cr, layout);
			
			length_line = length_line + length_maxi_quantity + 1;
			
			cairo_move_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (2 + 2 * i) * size_police);
			cairo_line_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (4 + 2 * i) * size_police);
		}
		
		cairo_move_to(cr, 0, HEADER_HEIGHT + HEADER_GAP + (4 + 2 * i) * size_police);
		cairo_line_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (4 + 2 * i) * size_police);
		
		number_article++;
  }
  
  /* Total */
  if(page_nr == pPrintData->total_pages - 1 && show_line_total){
	  gint NbArt, start_total;
  	gdouble PrixTot;
  	getNbArticlesAndPrixTotal(pPrintData->pGlda->pLdaChainee, pPrintData->pGlda->pMagasin, &NbArt, &PrixTot);
  	
  	length_line = 0;
  	
  	if(show_column_number){
  	length_line = length_line + length_maxi_number + 1;
  	}
  	
  	start_total = length_line;
  	
		cairo_move_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (2 + 2 * i) * size_police);
		cairo_line_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (4 + 2 * i) * size_police);
		
		if(show_column_name){
			cairo_move_to(cr, (length_line + 1) * size_police, HEADER_HEIGHT + HEADER_GAP + (2 + 2 * i) * size_police + cellpadding);
			pango_layout_set_text(layout, "Total", length_maxi_name);
			pango_cairo_show_layout(cr, layout);
			
			length_line = length_line + length_maxi_name + 1;
			
			cairo_move_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (2 + 2 * i) * size_police);
			cairo_line_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (4 + 2 * i) * size_police);
		}
		
		if(show_column_price){
			cairo_move_to(cr, (length_line + 1) * size_police, HEADER_HEIGHT + HEADER_GAP + (2 + 2 * i) * size_police + cellpadding);
			pText = g_strdup_printf("%4.2f %s", PrixTot, g_key_file_get_string(pPrintData->pGlda->pConfig, "general", "devise", NULL));
			pango_layout_set_text(layout, pText, length_maxi_price);
			g_free(pText);
			pango_cairo_show_layout(cr, layout);
			
			length_line = length_line + length_maxi_price + 1;
			
			cairo_move_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (2 + 2 * i) * size_police);
			cairo_line_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (4 + 2 * i) * size_police);
		}
		
		if(show_column_quantity){
			cairo_move_to(cr, (length_line + 1) * size_police, HEADER_HEIGHT + HEADER_GAP + (2 + 2 * i) * size_police + cellpadding);
			pText = g_strdup_printf("%d", NbArt);
			pango_layout_set_text(layout, pText, length_maxi_quantity);
			g_free(pText);
			pango_cairo_show_layout(cr, layout);
			
			length_line = length_line + length_maxi_quantity + 1;
			
			cairo_move_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (2 + 2 * i) * size_police);
			cairo_line_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (4 + 2 * i) * size_police);
		}
		
		cairo_move_to(cr, start_total * size_police, HEADER_HEIGHT + HEADER_GAP + (4 + 2 * i) * size_police);
		cairo_line_to(cr, length_line * size_police, HEADER_HEIGHT + HEADER_GAP + (4 + 2 * i) * size_police);
		/* Fin total */
	}
	
  cairo_stroke(cr);
  
  g_object_unref(layout);
  pango_font_description_free(desc);
}

void end_print(GtkPrintOperation *operation, GtkPrintContext *context, Print *pPrintData){
	
}
